library(poppr)
library(ade4)
library(adegenet)
library(data.table)
library(dplyr)

setwd("/path/to/AMOVA/")

# Implementation of distance changed from dissimilarity to Euclidean; use dissimilarity
V <- "2.6.1"
if( packageVersion("poppr") > V ) {
  url <- "https://cran.r-project.org/src/contrib/Archive/poppr/"
  system(paste0("curl ", url, "poppr_", V, ".tar.gz -o poppr_", V, ".tar.gz"))
  install.packages(paste0("poppr_", V, ".tar.gz"), type = "source", repos = NULL)
  .rs.restartR()
  library(poppr)
}

# Read in genotype and hierarchy files
DATA <- "data_radsnp_allparks.stru"
HIER <- "hier.txt"
y <- fread(DATA, sep = "\t", header = T) %>% 
  dplyr::select(-V1, -V2)
y <- read.structure(
  DATA,
  n.ind = nrow(y)/2,
  n.loc = ncol(y),
  onerowperind = FALSE,
  col.lab = 1,
  col.pop = 2,
  NA.char = "0",
  ask = F,
  row.marknames = 1
)
hier <- read.table(
  HIER, 
  header=T, 
  stringsAsFactors = F
) %>% 
  tibble() %>%
  dplyr::filter(Individual %in% row.names(y@tab)) %>% 
  dplyr::arrange( match(Individual, row.names(y@tab)) )

# Removing loci with missing data greater than X
y <- missingno(y, type = "loci", cutoff = 0.5)

# Define hierarchy
strata(y) <- hier %>% dplyr::select(Park,Lineage,Neighborhood,Meadow)

# Perform AMOVA
y <- as.genclone(y)
amova.all.PARKS <- 
  poppr.amova(
    x = y, 
    missing = "mean",
    hier = ~Park/Lineage/Neighborhood/Meadow
  )
amova.all.PARKS

# Bind together results
result.df <- cbind(
  amova.all.PARKS$results,
  amova.all.PARKS$componentsofcovariance,
  tibble(
    Phi.Name= row.names(amova.all.PARKS$statphi),
    amova.all.PARKS$statphi
  ) %>% 
    dplyr::arrange(desc(row_number())) %>%
    rbind(., tibble(
      Phi.Name = NA,
      Phi = NA
    ))
)
print(result.df)
write.table(result.df, "AMOVA.txt", sep = "\t", row.names = F, quote = F)

# Test for significance
amova.test <- randtest(amova.all.PARKS)
amova.test

# Plot result
plot(amova.test)

# Significance
alt <- c("less", "greater", "greater", "greater", "greater")
with(mon.test, as.krandtest(sim, obs, alter = alt, call = call, names = names))

# Island model calculation of migration rate (meadows)
idx.mdws.in.neig <- grepl("Between Meadow Within Neighborhood", row.names(result.df))
fst <- result.df$Phi[idx.mdws.in.neig]
fst_ind <- 1 / (4 * 29.13 * 0.1 + 1)
print(fst_ind)
print(fst)
fst > fst_ind

# Island model calculation of migration rate (neighborhoods)
idx.neigh.in.line <- grepl("Between Neighborhood Within Lineage", row.names(result.df))
fst <- result.df$Phi[idx.neigh.in.line]
fst_ind <- 1 / (4 * 23.33 * 0.1 + 1)
print(fst_ind)
print(fst)
fst > fst_ind

# Get pairwise Meadow within Neighborhood Phi-stats
x = y
hier = ~Neighborhood/Meadow
setPop(x) <- hier
hier <- update(hier, ~./Individual)
x <- poppr:::pool_haplotypes(x)
x <- missingno(x, type = "mean", cutoff = 0.05, quiet = FALSE)
hierdf <- strata(x, formula = hier)
xstruct <- poppr:::make_ade_df(hier, hierdf)
xdist <- sqrt(diss.dist(clonecorrect(x, strata = NA), percent = FALSE))
allmlgs <- unique(mlg.vector(x))
xtab <- t(mlg.table(x, plot = FALSE, quiet = TRUE, mlgsub = allmlgs))
xtab <- as.data.frame(xtab)

neighborhoods <- xstruct$Neighborhood %>% as.character() %>% unique()
nei.phi.pw.mean <- lapply(neighborhoods, function(n) {
  
  meadows <- x@strata %>% dplyr::filter(Neighborhood == n) %>% 
    pull(Meadow) %>% as.character() %>% unique()
  n.m <- length(meadows)
  
  if( n.m > 1 ) {
    meadow.pairs <- combn(n.m, 2) %>% t()
    PHI <- sapply(1:nrow(meadow.pairs), function(m) {
      M <- meadows[meadow.pairs[m,]]
      ind.n <- x@strata %>% dplyr::filter(Neighborhood %in% n & Meadow %in% M) %>% 
        dplyr::pull(Individual) %>% as.character() %>% unique()
      hap.n <- x@strata %>% dplyr::filter(Neighborhood %in% n & Meadow %in% M) %>% 
        rownames_to_column() %>% dplyr::pull(rowname) %>% as.character() %>% unique()
      xtab.n <- xtab[,colnames(xtab) %in% ind.n] %>% dplyr::mutate(sum = rowSums(.)) %>%
        dplyr::filter(sum == 1) %>% dplyr::select(-sum)
      xdist.n <- xdist %>% as.matrix()
      xdist.n <- xdist.n[rownames(xdist.n) %in% hap.n, colnames(xdist.n) %in% hap.n]
      xdist.n <- as.dist(xdist.n)
      hier = ~Meadow
      hier <- update(hier, ~./Individual)
      hierdf.n <- strata(x[x@pop %in% ind.n,], formula = hier)
      xstruct.n <- poppr:::make_ade_df(hier, hierdf.n)
      am <- ade4::amova(samples = xtab.n, distances = xdist.n, structures = xstruct.n)
      phi <- am$statphi %>% rownames_to_column() %>%
        dplyr::filter(rowname == "Phi-Meadow-total") %>% pull(Phi)
      return(phi)
    }) %>% mean()
  } else {
    PHI <- NA
  }
  
  tibble(
    Neighborhood = n,
    Phi = PHI
  )
}) %>% do.call(rbind, .)
nei.phi.pw.mean %>% print(n = 100)
write.table(nei.phi.pw.mean, "AMOVA_pw.txt", sep = "\t", quote = F, row.names = F)





